@@ -1,7 +1,7 @@
 #
 # coding=utf-8
 """versionah - Simple version specification management"""
-# Copyright © 2011, 2012  James Rowe <jnrowe@gmail.com>
+# Copyright © 2011-2014  James Rowe <jnrowe@gmail.com>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,7 +25,7 @@ from . import _version
 __version__ = _version.dotted
 __date__ = _version.date
 __author__ = 'James Rowe <jnrowe@gmail.com>'
-__copyright__ = 'Copyright © 2011, 2012, 2013  James Rowe <jnrowe@gmail.com>'
+__copyright__ = 'Copyright (C) 2011-2014  James Rowe <jnrowe@gmail.com>'
 __license__ = 'GNU General Public License Version 3'
 __credits__ = ''
 __history__ = 'See git repository'
@@ -56,6 +56,8 @@ import os
 import re
 import sys
 
+from functools import total_ordering
+
 try:
     # For Python 3
     from http.cookiejar import MONTHS
@@ -79,16 +81,10 @@ from .i18n import _
 T = Terminal()
 
 #: Base string type, used for compatibility with Python 2 and 3
-STR_TYPE = str if sys.version_info[0] == 3 else basestring
-
-#: Command line help string, for use with :mod:`argparse`
-# Pull the first paragraph from the docstring
-USAGE = '\n'.join(__doc__[:__doc__.find('\n\n', 100)].splitlines()[2:])
-# Replace script name with argparse's substitution var
-USAGE = USAGE.replace('versionah', '%(prog)s')
+STR_TYPE = basestring if sys.version_info[0] == 2 else str
 
 #: Command line interface object
-APP = aaargh.App(description=USAGE,
+APP = aaargh.App(description=_('A tool to manage project version files'),
                  epilog=_('Please report bugs to jnrowe@gmail.com'))
 
 #: Regular expression to match a valid package name
@@ -164,6 +160,7 @@ def filter_regexp(string, pattern, repl, count=0, flags=0):
 FILTERS['regexp'] = filter_regexp
 
 
+@total_ordering
 class Version(object):
 
     """Main version identifier representation."""
@@ -267,7 +264,6 @@ class Version(object):
 
         """
         return self.components_full == self.__prepare_cmp_object(other)
-    __ne__ = lambda self, other: not self == (other)
 
     def __lt__(self, other):
         """Strict less-than test against comparable object.
@@ -280,39 +276,6 @@ class Version(object):
         """
         return self.components < self.__prepare_cmp_object(other)
 
-    def __gt__(self, other):
-        """Strict greater-than test against comparable object.
-
-        See `~Version.__prepare_cmp_object`.
-
-        :rtype: `bool`
-        :return: True if ``self`` is strictly greater-than ``other``
-
-        """
-        return self.components_full > self.__prepare_cmp_object(other)
-
-    def __le__(self, other):
-        """Less-than or equal to test against comparable object.
-
-        See `~Version.__prepare_cmp_object`.
-
-        :rtype: `bool`
-        :return: True if ``self`` is less-than or equal to ``other``
-
-        """
-        return self < other or self == other
-
-    def __ge__(self, other):
-        """Greater-than or equal to test against comparable object.
-
-        See `~Version.__prepare_cmp_object`.
-
-        :rtype: `bool`
-        :return: True if ``self`` is greater-than or equal to ``other``
-
-        """
-        return self > other or self == other
-
     def __hash__(self):
         """Create object-unique hash value.
 
@@ -363,7 +326,7 @@ class Version(object):
 
         """
         if bump_type == 'micro' and self._resolution < 3 \
-            or bump_type == 'patch' and self._resolution < 4:
+                or bump_type == 'patch' and self._resolution < 4:
             raise ValueError('Invalid bump_type %r for version %r'
                              % (bump_type, self.components))
         if bump_type == 'major':
@@ -397,6 +360,15 @@ class Version(object):
         """Bump patch version component."""
         self.bump('patch')
 
+    def as_dict(self):
+        """Generate a dictionary of version components.
+
+        :rtype: `dict`
+        :return: Version as dictionary
+
+        """
+        return dict(zip(['major', 'minor', 'micro', 'patch'], self.components))
+
     def as_dotted(self):
         """Generate a dotted version string.
 
@@ -478,11 +450,12 @@ class Version(object):
         :param str filename: Version file to read
         :rtype: `Version`
         :return: New `Version` object representing file
-        :raise OSError: When ``filename`` doesn't exist
-        :raise ValueError: Unparsable version data
+        :raise exceptions.OSError: When ``filename`` doesn't exist
+        :raise exceptions.ValueError: Unparsable version data
 
         """
-        data = open(filename).read().strip()
+        with open(filename) as f:
+            data = f.read().strip()
         match = re.search(r'This is (%s),? [vV]ersion (%s) \((%s)\)'
                           % (VALID_PACKAGE, VALID_VERSION, VALID_DATE),
                           data)
@@ -496,14 +469,17 @@ class Version(object):
             parsed = datetime.datetime.strptime(date_str, '%d-%b-%Y')
         return Version(components, name, parsed.date())
 
-    def write(self, filename, file_type):
+    def write(self, filename, file_type, shtool=False):
         """Write a version file.
 
         :param str filename: Version file to write
         :param str file_type: File type to write
+        :param bool shtool: Write shtool_ compatible files
         :rtype: `bool`
         :return: `True` on write success
 
+        .. _shtool: http://www.gnu.org/software/shtool/shtool.html
+
         """
         data = vars(self)
         data.update({
@@ -512,17 +488,28 @@ class Version(object):
             'filename': filename,
             'dateobj': self.date,
             'resolution': self._resolution,
-            'magic': 'This is %s version %s (%s)' % (self.name,
-                                                     self.as_dotted(),
-                                                     self.as_date()),
         })
+        if shtool:
+            # %d-%b-%Y, if %b wasn't locale dependent
+            shtool_date = "%s-%s-%s" % (self.date.day,
+                                        MONTHS[self.date.month-1],
+                                        self.date.year)
+            data['magic'] = 'This is %s, Version %s (%s)' % (self.name,
+                                                             self.as_dotted(),
+                                                             shtool_date)
+        else:
+            data['magic'] = 'This is %s version %s (%s)' % (self.name,
+                                                            self.as_dotted(),
+                                                            self.as_date())
+
         data.update(dict(zip(['major', 'minor', 'micro', 'patch'],
                              self.components)))
         data.update(dict([(k[3:], getattr(self, k)())
                           for k in dir(self) if k.startswith('as_')]))
 
         template = self.env.get_template('%s.jinja' % file_type)
-        open(filename, 'w').write(template.render(data))
+        with open(filename, 'w') as f:
+            f.write(template.render(data))
 
 
 def split_version(version):
@@ -531,7 +518,7 @@ def split_version(version):
     :param str version: Version string
     :rtype: `tuple` of `int`
     :return: Components of version string
-    :raise ValueError: Invalid version string
+    :raise exceptions.ValueError: Invalid version string
 
     """
     if not re.match('%s$' % VALID_VERSION, version):
@@ -561,20 +548,26 @@ OPTIONS.add_argument('-d', '--display', default='dotted',
 OPTIONS.add_argument('filename', help=_('version file to operate on'))
 
 
-@APP.cmd(help=_('bump version in given file'), parents=[OPTIONS, ])
+@APP.cmd(name='bump', help=_('bump version in given file'),
+         parents=[OPTIONS, ])
 @APP.cmd_arg('-t', '--type', choices=Version.filetypes, dest='file_type',
              metavar='text',
              help=_('define the file type used for version file'))
-@APP.cmd_arg('bump', choices=('major', 'minor', 'micro', 'patch'),
-             help=_('bump type'))
-def bump(display_format, filename, file_type, bump):
+@APP.cmd_arg('--shtool', action='store_true',
+             help=_('write shtool compatible output'))
+@APP.cmd_arg('bump', default=None, nargs='?',
+             choices=('major', 'minor', 'micro', 'patch'), help=_('bump type'))
+def bump_version(display_format, filename, file_type, shtool, bump):
     """Bump version in existing file.
 
     :param str display_format: Format to display output in
     :param str filename: File to operate on
     :param str file_type: File type to produce
+    :param bool shtool: Write shtool_ compatible files
     :param str bump: Component to bump
 
+    .. _shtool: http://www.gnu.org/software/shtool/shtool.html
+
     """
     if not file_type:
         file_type = guess_type(filename)
@@ -589,13 +582,16 @@ def bump(display_format, filename, file_type, bump):
         print(fail(_('File not found')))
         return errno.ENOENT
 
+    if not bump:
+        bump = ('major', 'minor', 'micro', 'patch')[len(version.components) - 1]
+
     version.bump(bump)
-    version.write(filename, file_type)
+    version.write(filename, file_type, shtool)
 
     print(success(version.display(display_format)))
 
 
-@APP.cmd(help=_('set version in given file'), parents=[OPTIONS, ])
+@APP.cmd(name='set', help=_('set version in given file'), parents=[OPTIONS, ])
 @APP.cmd_arg('-t', '--type', choices=Version.filetypes, dest='file_type',
              metavar='text',
              help=_('define the file type used for version file'))
@@ -603,16 +599,16 @@ def bump(display_format, filename, file_type, bump):
              metavar=os.path.basename(os.getenv('PWD')),
              action=ValidatingAction,
              help=_('package name for version(default from $PWD)'))
-@APP.cmd_arg('set', metavar='0.1.0', action=ValidatingAction,
+@APP.cmd_arg('version_str', metavar='version', action=ValidatingAction,
              help='set to a specific version')
-def set(display_format, filename, file_type, name, set):
+def set_version(display_format, filename, file_type, name, version_str):
     """Set version in new or existing file.
 
     :param str display_format: Format to display output in
     :param str filename: File to operate on
     :param str file_type: File type to produce
     :param str name: Project name used in output
-    :param str set: Initial version string
+    :param str version_str: Initial version string
 
     """
     if not file_type:
@@ -629,7 +625,7 @@ def set(display_format, filename, file_type, name, set):
     if name:
         version.name = name
 
-    version.set(set)
+    version.set(version_str)
     version.write(filename, file_type)
 
     print(success(version.display(display_format)))
@@ -648,6 +644,9 @@ def display(display_format, filename):
     except IOError as error:
         print(fail(error.args[1]))
         return errno.EEXIST
+    except ValueError as error:
+        print(fail(error.args[0]))
+        return errno.EIO
 
     print(success(version.display(display_format)))
 
@@ -657,4 +656,4 @@ def main():
 
     APP.arg('--version', action='version',
             version='%%(prog)s %s' % _version.dotted)
-    APP.run()
+    return APP.run()
